@@ -25,6 +25,7 @@ from ludwig.automl.utils import (
     _ray_init,
     get_available_resources,
     get_model_type,
+    has_imbalanced_output,
     set_output_feature_metric,
 )
 from ludwig.constants import (
@@ -107,12 +108,13 @@ def auto_train(
                         parameter initialization and training set shuffling
     :param use_reference_config: (bool) refine hyperopt search space by setting first
                                  search point from reference model config, if any
+    :param kwargs: additional keyword args passed down to `ludwig.hyperopt.run.hyperopt`.
 
     # Returns
     :return: (AutoTrainResults) results containing hyperopt experiments and best model
     """
     config = create_auto_config(
-        dataset, target, time_limit_s, tune_for_memory, user_config, random_seed, use_reference_config, **kwargs
+        dataset, target, time_limit_s, tune_for_memory, user_config, random_seed, use_reference_config
     )
     return train_with_config(dataset, config, output_directory=output_directory, random_seed=random_seed, **kwargs)
 
@@ -147,8 +149,10 @@ def create_auto_config(
     # Return
     :return: (dict) selected model configuration
     """
-    default_configs = _create_default_config(dataset, target, time_limit_s, random_seed)
-    model_config, model_category, row_count = _model_select(dataset, default_configs, user_config, use_reference_config)
+    default_configs, features_metadata = _create_default_config(dataset, target, time_limit_s, random_seed)
+    model_config, model_category, row_count = _model_select(
+        dataset, default_configs, features_metadata, user_config, use_reference_config
+    )
     if tune_for_memory:
         if ray.is_initialized():
             resources = get_available_resources()  # check if cluster has GPUS
@@ -191,6 +195,7 @@ def train_with_config(
                         there is a call to a random number generator, including
                         hyperparameter search sampling, as well as data splitting,
                         parameter initialization and training set shuffling
+    :param kwargs: additional keyword args passed down to `ludwig.hyperopt.run.hyperopt`.
 
     # Returns
     :return: (AutoTrainResults) results containing hyperopt experiments and best model
@@ -218,6 +223,7 @@ def train_with_config(
 def _model_select(
     dataset: Union[str, pd.DataFrame, dd.core.DataFrame, DatasetInfo],
     default_configs,
+    features_metadata,
     user_config,
     use_reference_config: bool,
 ):
@@ -271,6 +277,10 @@ def _model_select(
                 if param in user_config[config_section]:
                     del base_config["hyperopt"]["parameters"][hyperopt_params]
 
+    # check if any binary or category output feature has highly imbalanced minority vs majority values
+    # note: check is done after any relevant user_config has been applied
+    has_imbalanced_output(base_config, features_metadata)
+
     # if single output feature, set relevant metric and goal if not already set
     base_config = set_output_feature_metric(base_config)
 
@@ -297,7 +307,6 @@ def _train(
         output_directory=output_directory,
         model_name=model_name,
         random_seed=random_seed,
-        backend="local",
         skip_save_log=True,  # avoid per-step log overhead by default
         **kwargs,
     )
