@@ -58,6 +58,7 @@ from ludwig.utils.data_utils import (
     CACHEABLE_FORMATS,
     CSV_FORMATS,
     DATA_TRAIN_HDF5_FP,
+    DATA_TRAIN_PARQUET_FP,
     DATAFRAME_FORMATS,
     DICT_FORMATS,
     EXCEL_FORMATS,
@@ -558,6 +559,8 @@ class ParquetPreprocessor(DataFormatPreprocessor):
     ):
         test_set = test_set if test_set and path_exists(test_set) else None
         validation_set = validation_set if validation_set and path_exists(validation_set) else None
+        if training_set and isinstance(training_set, str) and DATA_TRAIN_PARQUET_FP not in training_set_metadata:
+            training_set_metadata[DATA_TRAIN_PARQUET_FP] = training_set
         return training_set, test_set, validation_set, training_set_metadata
 
 
