@@ -24,9 +24,9 @@ import ray
 import torch
 
 from ludwig.api import LudwigModel
-from ludwig.backend import create_ray_backend, LOCAL_BACKEND
+from ludwig.backend import create_ray_backend, initialize_backend, LOCAL_BACKEND
 from ludwig.backend.ray import get_trainer_kwargs, RayBackend
-from ludwig.constants import BACKFILL, BALANCE_PERCENTAGE_TOLERANCE, NAME, TRAINER
+from ludwig.constants import BACKFILL, BALANCE_PERCENTAGE_TOLERANCE, COLUMN, NAME, TRAINER
 from ludwig.data.dataframe.dask import DaskEngine
 from ludwig.data.preprocessing import balance_data
 from ludwig.utils.data_utils import read_parquet
@@ -43,7 +43,6 @@ from tests.integration_tests.utils import (
     number_feature,
     sequence_feature,
     set_feature,
-    spawn,
     text_feature,
     timeseries_feature,
     train_with_backend,
@@ -147,7 +146,6 @@ def split(data_parquet):
     return train_fname, val_fname, test_fname
 
 
-@spawn
 def run_test_with_features(
     input_features,
     output_features,
@@ -195,6 +193,46 @@ def run_test_with_features(
                 )
 
 
+@pytest.mark.parametrize("df_engine", ["pandas", "dask"])
+@pytest.mark.distributed
+def test_ray_read_binary_files(tmpdir, df_engine):
+    preprocessing_params = {
+        "audio_file_length_limit_in_s": 3.0,
+        "missing_value_strategy": BACKFILL,
+        "in_memory": True,
+        "padding_value": 0,
+        "norm": "per_file",
+        "audio_feature": {
+            "type": "fbank",
+            "window_length_in_s": 0.04,
+            "window_shift_in_s": 0.02,
+            "num_filter_bands": 80,
+        },
+    }
+    audio_dest_folder = os.path.join(tmpdir, "generated_audio")
+    audio_params = audio_feature(folder=audio_dest_folder, preprocessing=preprocessing_params)
+
+    dataset_path = os.path.join(tmpdir, "dataset.csv")
+    dataset_path = generate_data([audio_params], [], dataset_path, num_examples=100)
+    dataset_path = create_data_set_to_use("csv", dataset_path, nan_percent=0.1)
+
+    with ray_start(num_cpus=2, num_gpus=None):
+        backend_config = {**RAY_BACKEND_CONFIG}
+        backend_config["processor"]["type"] = df_engine
+        backend = initialize_backend(backend_config)
+        df = backend.df_engine.df_lib.read_csv(dataset_path)
+        series = df[audio_params[COLUMN]]
+        proc_col = backend.read_binary_files(series)
+        proc_col = backend.df_engine.compute(proc_col)
+
+        backend = initialize_backend(LOCAL_BACKEND)
+        df = backend.df_engine.df_lib.read_csv(dataset_path)
+        series = df[audio_params[COLUMN]]
+        proc_col_expected = backend.read_binary_files(series)
+
+        assert proc_col.equals(proc_col_expected)
+
+
 @pytest.mark.parametrize("dataset_type", ["csv", "parquet"])
 @pytest.mark.distributed
 def test_ray_save_processed_input(dataset_type):
@@ -260,9 +298,10 @@ def test_ray_sequence():
     run_test_with_features(input_features, output_features)
 
 
-@pytest.mark.parametrize("feature_type", ["raw", "stft", "stft_phase", "group_delay", "fbank"])
+@pytest.mark.parametrize("dataset_type", ["csv", "parquet"])
+@pytest.mark.parametrize("feature_type", ["raw", "fbank"])
 @pytest.mark.distributed
-def test_ray_audio(feature_type):
+def test_ray_audio(dataset_type, feature_type):
     with tempfile.TemporaryDirectory() as tmpdir:
         preprocessing_params = {
             "audio_file_length_limit_in_s": 3.0,
@@ -280,7 +319,7 @@ def test_ray_audio(feature_type):
         audio_dest_folder = os.path.join(tmpdir, "generated_audio")
         input_features = [audio_feature(folder=audio_dest_folder, preprocessing=preprocessing_params)]
         output_features = [binary_feature()]
-        run_test_with_features(input_features, output_features)
+        run_test_with_features(input_features, output_features, dataset_type=dataset_type)
 
 
 @pytest.mark.distributed
@@ -462,3 +501,19 @@ def test_tune_batch_size_lr():
             model = run_api_experiment(config, dataset=dataset_parquet, backend_config=backend_config)
             assert model.config[TRAINER]["batch_size"] != "auto"
             assert model.config[TRAINER]["learning_rate"] != "auto"
+
+
+@pytest.mark.distributed
+def test_ray_progress_bar():
+    # This is a simple test that is just meant to make sure that the progress bar isn't breaking
+    input_features = [
+        sequence_feature(reduce_output="sum"),
+    ]
+    output_features = [
+        binary_feature(bool2str=["No", "Yes"]),
+    ]
+    run_test_with_features(
+        input_features,
+        output_features,
+        df_engine="dask",
+    )
