@@ -24,7 +24,7 @@ import traceback
 import unittest
 import uuid
 from distutils.util import strtobool
-from typing import List, Union
+from typing import List
 
 import cloudpickle
 import numpy as np
@@ -39,7 +39,6 @@ from ludwig.data.dataset_synthesizer import build_synthetic_dataset, DATETIME_FO
 from ludwig.experiment import experiment_cli
 from ludwig.features.feature_utils import compute_feature_hash
 from ludwig.models.trainer import Trainer
-from ludwig.utils import image_utils
 from ludwig.utils.data_utils import read_csv, replace_file_extension
 
 logger = logging.getLogger(__name__)
@@ -319,17 +318,6 @@ def vector_feature(**kwargs):
     return feature
 
 
-def to_inference_module_input(s: pd.Series) -> Union[List[str], List[torch.Tensor], torch.Tensor]:
-    """Converts a pandas Series to be compatible with a torchscripted InferenceModule forward pass."""
-    if "image" in s.name:
-        return [image_utils.read_image(v) for v in s]
-    if s.dtype == "object":
-        if "bag" in s.name or "set" in s.name:
-            s = s.astype(str)
-        return s.to_list()
-    return torch.from_numpy(s.to_numpy())
-
-
 def run_experiment(
     input_features=None, output_features=None, config=None, skip_save_processed_input=True, backend=None, **kwargs
 ):
@@ -525,9 +513,11 @@ def read_csv_with_nan(path, nan_percent=0.0):
     df = pd.read_csv(path)
     if nan_percent > 0:
         num_rows = len(df)
+        num_nans_per_col = int(round(nan_percent * num_rows))
         for col in df.columns:
-            for row in random.sample(range(num_rows), int(round(nan_percent * num_rows))):
-                df[col].iloc[row] = np.nan
+            col_idx = df.columns.get_loc(col)
+            for row_idx in random.sample(range(num_rows), num_nans_per_col):
+                df.iloc[row_idx, col_idx] = np.nan
     return df
 
 
